import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import Graph, { DirectedGraph } from 'graphology';

/*************** schema format from the backend *************** */
// TODO: should probably not live here
/** Schema type, consist of nodes and edges */
export type SchemaFromBackend = {
  edges: Edge[];
  nodes: Node[];
};

/** Attribute type, consist of a name */
export type Attribute = {
  name: string;
  type: 'string' | 'int' | 'bool' | 'float';
};

/** Node type, consist of a name and a list of attributes */
export type Node = {
  name: string;
  attributes: Attribute[];
};

/** Edge type, consist of a name, start point, end point and a list of attributes */
export type Edge = {
  name: string;
  to: string;
  from: string;
  collection: string;
  attributes: Attribute[];
};
/**************************************************************** */

// Define the initial state using that type
export const initialState = {
  graphologySerialized: new Graph().export(),
};

export const schemaSlice = createSlice({
  name: 'schema',
  // `createSlice` will infer the state type from the `initialState` argument
  initialState,
  reducers: {
    readInSchemaFromBackend: (
      state,
      action: PayloadAction<SchemaFromBackend>
    ) => {
      const { nodes, edges } = action.payload;
      // Instantiate a directed graph that allows self loops and parallel edges
      const schema = new DirectedGraph({ allowSelfLoops: true, multi: true });
      nodes.forEach((node) => {
        schema.addNode(node.name, { attributes: node.attributes });
      });

      edges.forEach((edge) =>
        schema.addDirectedEdgeWithKey(edge.name, edge.from, edge.to, {
          collection: edge.collection,
        })
      );

      state.graphologySerialized = schema.export();
    },
  },
});

export const { readInSchemaFromBackend } = schemaSlice.actions;

// Select the schema and convert it to a graphology object
export const selectSchema = (state: RootState) =>
  Graph.from(state.schema.graphologySerialized);

export default schemaSlice.reducer;
